﻿using CredentialManagement;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace RCM {
    [DisplayName("Start Remote Desktop")]
    public class RDPAction : IAction {
        private RDP _rdp;

        public RDPAction(RDP rdp) {
            _rdp = rdp;
        }

        public async void Run() {
            await Task.Run(() => {
                if (Environment.OSVersion.Platform == PlatformID.Unix) {
                    RunUnix();
                } else {
                    RunWin();
                }
            });
        }

        private void RunWin() {
            string[] host = _rdp.Host.Split(new char[] { ':' });
            string[] username = _rdp.Username.Split(new char[] { '\\' });

            // https://docs.microsoft.com/en-us/previous-versions/windows/it-pro/windows-server-2008-R2-and-2008/ff393699(v%3dws.10)
            string rdpFile = Path.GetTempPath() + Guid.NewGuid().ToString() + ".rdp";
            using (StreamWriter sw = new StreamWriter(rdpFile)) {
                sw.Write("drivestoredirect:s:*\r\nredirectprinters:i:0\r\nredirectcomports:i:0\r\nredirectsmartcards:i:0\r\nredirectposdevices:i:0\r\nnetworkautodetect:i:0\r\nauthentication level:i:0\r\nprompt for credentials:i:0\r\nnegotiate security layer:i:1\r\npromptcredentialonce:i:0\r\nsmart sizing:i:0\r\nuse multimon:i:0\r\nbandwidthautodetect:i:1\r\n");
                sw.Write(string.Format("full address:s:{0}\r\n", _rdp.Host));
                sw.Write(string.Format("username:s:{0}\r\n", _rdp.Username));
                if (username.Length == 2) {
                    sw.Write(string.Format("domain:s:{0}\r\n", username[0]));
                }
            }
            Credential credential = new Credential(_rdp.Username, _rdp.Password, string.Format("TERMSRV/{0}", host[0]));
            credential.Save();
            Process p = Process.Start("mstsc", string.Format("\"{0}\"", rdpFile));
            Thread.Sleep(1000);
            File.Delete(rdpFile);
        }

        private void RunUnix() {
            // TODO
        }
    }
}
